"""Function used to extract package information from a local setup.py"""
import os
import sys


def extract_setup_metadata():
    """Get all values of parameters passed to setup()"""
    import distutils.core
    import setuptools

    package_metadata = {}

    def on_setup_loaded(**metadata):
        package_metadata.update(metadata)

    # mock setup functions to only print package name
    setuptools.setup = on_setup_loaded
    distutils.core.setup = on_setup_loaded

    # make sure we will be able to import setup from current directory
    current_directory = os.getcwd()
    sys.path.insert(0, current_directory)

    # import you setup.py with mocked setup()
    print('Loading setup from directory', current_directory, file=sys.stderr)
    import setup

    return package_metadata


def get_setup_metadata(key: str):
    """Get value of a specific parameter of setup()"""
    package_metadata = extract_setup_metadata()
    value = package_metadata[key]
    print(f'Value for key {key} in setup is {value}', file=sys.stderr)
    return value



import re

import requests

from pyrman.pypi import parsers

ABSOLUTE_URL_PATTERN = re.compile(r"^https?://")

DEFAULT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0",
    "authority": "pypi.org",
    "origin": "https://pypi.org",
}


class PyPISession(requests.Session):
    def __init__(self, base_url: str):
        super().__init__()
        self.headers.update(DEFAULT_HEADERS)
        self.base_url = base_url.rstrip("/")
        #
        self.username = None
        self.password = None
        self.is_authenticated = False

    def absolute_url(self, url: str):
        if ABSOLUTE_URL_PATTERN.match(url):
            return url
        return self.base_url + "/" + url.lstrip("/")

    def request(self, method: str, url: str, **options):
        # authenticate if needed
        if not self.is_authenticated and not "/account/login/" in url:
            self.authenticate(self.username, self.password)
        return super().request(method, self.absolute_url(url), **options)

    def get_login_csrf_token(self) -> str:
        response = self.get("/account/login/")
        response.raise_for_status()
        return parsers.parse_csrf_token(response.text)

    def set_credentials(self, username: str, password: str):
        self.username = username
        self.password = password

    def authenticate(self, username: str, password: str):
        csrf_token = self.get_login_csrf_token()
        response = self.post(
            "/account/login/",
            headers={"referer": f"{self.base_url}/account/login/",},
            data={"csrf_token": csrf_token, "username": username, "password": password,},
            allow_redirects=False,
        )
        response.raise_for_status()

        redirect = response.headers.get("location")
        if not redirect or "/account/login/" in redirect:
            raise ValueError("Failed to authenticate")



from dataclasses import dataclass
from datetime import datetime


@dataclass
class Package:
    title: str
    description: str
    last_released: datetime
    package_url: str
    releases_url: str


@dataclass
class Release:
    version: str
    release_date: datetime



import setuptools


def get_file_content(filename: str):
    with open(filename, "r", encoding="utf-8") as file:
        return file.read()


setuptools.setup(
    name="releases-manager",
    description="Manage python published release for packages that you owns on PyPI.",
    version="0.0.5",
    packages=setuptools.find_packages(),
    install_requires=["bs4", "click", "pydantic", "pyyaml", "requests"],
    entry_points={"console_scripts": ["pyrman=pyrman.__main__:cli"]},
    long_description=get_file_content("README.md"),
    long_description_content_type="text/markdown",
)



import dataclasses
from datetime import datetime

import yaml


def make_serializable(obj):
    if dataclasses.is_dataclass(obj):
        return dataclasses.asdict(obj)
    elif isinstance(obj, (list, tuple)):
        return [dataclasses.asdict(item) for item in obj]
    elif isinstance(obj, dict):
        return {key: dataclasses.asdict(value) for key, value in obj.items()}
    else:
        return obj


def serialize_as_yaml(obj):
    return yaml.safe_dump(make_serializable(obj)).replace("- ", "\n- ")
